"""Order types for Carlo TMS integration based on FOneOrderImport-v3.xsd.

This module provides Pydantic models for Carlo order import/export functionality,
similar to the LisInAuftrag pattern used in lis_winsped.
"""

from typing import Optional, List, Any, Tuple
from datetime import datetime
from datetime import date as date_type
from decimal import Decimal
import xml.etree.ElementTree as ET
from pathlib import Path
from pydantic import BaseModel, Field
import uuid
from lxml import etree as lxml_etree

from .types.enums import ImportExportAction, OrderContext
from .types.base import (
    Header,
    OrganizationalUnit,
    Currency,
)
from .types.business_partner import BusinessPartner, ContactPerson
from .types.consignment import Consignment


class Order(BaseModel):
    """The order."""

    date: Optional[date_type] = Field(None, serialization_alias="Date")
    object_owner: Optional[OrganizationalUnit] = Field(
        None, serialization_alias="ObjectOwner"
    )
    number: Optional[int] = Field(None, serialization_alias="Number")
    external_number: Optional[str] = Field(
        None, max_length=50, serialization_alias="ExternalNumber"
    )
    order_context: Optional[OrderContext] = Field(
        None, serialization_alias="OrderContext"
    )
    order_date: Optional[date_type] = Field(None, serialization_alias="OrderDate")
    order_reference1: Optional[str] = Field(
        None, max_length=256, serialization_alias="OrderReference1"
    )
    order_reference2: Optional[str] = Field(
        None, max_length=256, serialization_alias="OrderReference2"
    )
    order_reference3: Optional[str] = Field(
        None, max_length=256, serialization_alias="OrderReference3"
    )
    info: Optional[str] = Field(None, max_length=2000, serialization_alias="Info")
    customer: Optional[BusinessPartner] = Field(None, serialization_alias="Customer")
    customer_contact_person: Optional[ContactPerson] = Field(
        None, serialization_alias="CustomerContactPerson"
    )
    freight_payer: Optional[BusinessPartner] = Field(
        None, serialization_alias="FreightPayer"
    )
    consignments: Optional[List[Consignment]] = Field(
        None, serialization_alias="Consignments"
    )
    custom_integer1: Optional[int] = Field(None, serialization_alias="CustomInteger1")
    custom_integer2: Optional[int] = Field(None, serialization_alias="CustomInteger2")
    custom_integer3: Optional[int] = Field(None, serialization_alias="CustomInteger3")
    custom_integer4: Optional[int] = Field(None, serialization_alias="CustomInteger4")
    custom_float1: Optional[Decimal] = Field(None, serialization_alias="CustomFloat1")
    custom_float2: Optional[Decimal] = Field(None, serialization_alias="CustomFloat2")
    custom_float3: Optional[Decimal] = Field(None, serialization_alias="CustomFloat3")
    custom_float4: Optional[Decimal] = Field(None, serialization_alias="CustomFloat4")
    custom_float5: Optional[Decimal] = Field(None, serialization_alias="CustomFloat5")
    custom_date_time1: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime1"
    )
    custom_date_time2: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime2"
    )
    custom_date_time3: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime3"
    )
    custom_date_time4: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime4"
    )
    custom_date_time5: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime5"
    )
    custom_date_time6: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime6"
    )
    is_empty_trip: Optional[bool] = Field(None, serialization_alias="IsEmptyTrip")
    posting_reference1: Optional[str] = Field(
        None, max_length=30, serialization_alias="PostingReference1"
    )
    posting_reference2: Optional[str] = Field(
        None, max_length=30, serialization_alias="PostingReference2"
    )
    posting_reference3: Optional[str] = Field(
        None, max_length=30, serialization_alias="PostingReference3"
    )
    posting_reference4: Optional[str] = Field(
        None, max_length=30, serialization_alias="PostingReference4"
    )
    vessel_exchange_currency: Optional[Currency] = Field(
        None, serialization_alias="VesselExchangeCurrency"
    )
    vessel_exchange_rate: Optional[Decimal] = Field(
        None, serialization_alias="VesselExchangeRate"
    )
    vessel_exchange_date: Optional[date_type] = Field(
        None, serialization_alias="VesselExchangeDate"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class OrderData(BaseModel):
    """Collection of orders - root element for XML import."""

    header: Optional[Header] = Field(None, serialization_alias="Header")
    orders: List[Order] = Field(
        default_factory=list[Order], serialization_alias="Order"
    )

    model_config = {"populate_by_name": True}


class NormalOrderData(BaseModel):
    """
    Main class for Carlo order data similar to LisInAuftrag.

    This class provides validation and XML generation functionality for
    Carlo TMS order import.

    Usage:
        ```python
        from tms_integration.carlo.models import NormalOrderData, Order, Consignment

        order_data = NormalOrderData(
            orders=[
                Order(
                    date=date.today(),
                    customer=BusinessPartner(number=123456),
                    consignments=[
                        Consignment(
                            item_number=1,
                            sender=ConsignmentBusinessPartner(
                                name1="Sender Company",
                                city1="Berlin",
                                country="DE"
                            ),
                            receiver=ConsignmentBusinessPartner(
                                name1="Receiver Company",
                                city1="Munich",
                                country="DE"
                            )
                        )
                    ]
                )
            ]
        )

        xml_string = order_data.generate_xml()
        ```
    """

    header: Optional[Header] = Field(None, serialization_alias="Header")
    orders: List[Order] = Field(
        default_factory=list[Order], serialization_alias="Order"
    )

    model_config = {"populate_by_name": True}

    def validate_orders(self) -> None:
        """
        Validate that the order data has required fields.

        Raises:
            ValueError: If validation fails (e.g., no orders provided)
        """
        if not self.orders:
            raise ValueError("At least one order is required")

        for i, order in enumerate(self.orders):
            if order.consignments:
                for j, consignment in enumerate(order.consignments):
                    if not consignment.sender and not consignment.receiver:
                        raise ValueError(
                            f"Order {i + 1}, Consignment {j + 1}: "
                            "At least sender or receiver is required"
                        )

    def _model_to_xml_element(self, tag: str, model: BaseModel) -> ET.Element:
        """Convert a Pydantic model to an XML element using serialization aliases."""
        element = ET.Element(tag)

        for field_name, field_info in type(model).model_fields.items():
            # Get the serialization alias or use the field name
            alias = field_info.serialization_alias or field_name
            value: Any = getattr(model, field_name)

            if value is None:
                continue

            if isinstance(value, BaseModel):
                child = self._model_to_xml_element(alias, value)
                element.append(child)
            elif isinstance(value, list):
                for item in value:  # type: ignore
                    if isinstance(item, BaseModel):
                        # For list items, use the alias as the tag
                        # (e.g., Consignments -> each item is Consignments)
                        child = self._model_to_xml_element(alias, item)
                        element.append(child)
                    else:
                        child = ET.SubElement(element, alias)
                        child.text = self._format_value(item)
            elif isinstance(value, datetime):
                child = ET.SubElement(element, alias)
                child.text = value.strftime("%Y-%m-%dT%H:%M:%S")
            elif isinstance(value, date_type):
                child = ET.SubElement(element, alias)
                child.text = value.strftime("%Y-%m-%d")
            elif isinstance(value, bool):
                child = ET.SubElement(element, alias)
                child.text = "true" if value else "false"
            elif isinstance(value, (int, float, Decimal)):
                child = ET.SubElement(element, alias)
                child.text = str(value)
            else:
                child = ET.SubElement(element, alias)
                child.text = str(value)

        return element

    def _format_value(self, value: Any) -> str:
        """Format a value for XML output."""
        if isinstance(value, datetime):
            return value.strftime("%Y-%m-%dT%H:%M:%S")
        elif isinstance(value, date_type):
            return value.strftime("%Y-%m-%d")
        elif isinstance(value, bool):
            return "true" if value else "false"
        else:
            return str(value)

    def generate_xml(self, validate: bool = True) -> str:
        """
        Generate XML string from the order data.

        Args:
            validate: If True, validate the order data before generating XML.
                     Defaults to True.

        Returns:
            str: XML string representation of the order data.

        Raises:
            ValueError: If validation is enabled and fails.
        """
        if validate:
            self.validate_orders()

        # Create root element with namespace
        root = ET.Element("OrderData")
        root.set("xmlns", "SoloplanOrderImport.v2")

        # Add Header if present, or create a default one
        if self.header:
            header_element = self._model_to_xml_element("Header", self.header)
        else:
            header_element = ET.SubElement(root, "Header")
            send_date = ET.SubElement(header_element, "SendDate")
            send_date.text = datetime.now().strftime("%Y-%m-%dT%H:%M:%S")
            export_ref = ET.SubElement(header_element, "ExportItemReference")
            export_ref.text = str(uuid.uuid4())

        if self.header:
            root.append(header_element)

        # Add Orders
        for order in self.orders:
            order_element = self._model_to_xml_element("Order", order)
            root.append(order_element)

        # Generate XML string with declaration
        xml_declaration = '<?xml version="1.0" encoding="utf-8"?>\n'
        xml_body = ET.tostring(root, encoding="unicode")

        return xml_declaration + xml_body

    @staticmethod
    def get_xsd_path() -> Path:
        """
        Get the path to the XSD schema file.

        Returns:
            Path: Path to FOneOrderImport-v3.xsd
        """
        return Path(__file__).parent / "xsd" / "FOneOrderImport-v3.xsd"

    def validate_against_xsd(
        self, xml_string: Optional[str] = None
    ) -> Tuple[bool, List[str]]:
        """
        Validate the XML against the XSD schema.

        Args:
            xml_string: Optional XML string to validate. If not provided,
                       generates XML from the current order data.

        Returns:
            Tuple[bool, List[str]]: A tuple containing:
                - bool: True if validation passed, False otherwise
                - List[str]: List of validation error messages (empty if valid)

        Raises:
            ImportError: If lxml is not installed
            FileNotFoundError: If the XSD file is not found

        Example:
            ```python
            order_data = NormalOrderData(orders=[...])
            is_valid, errors = order_data.validate_against_xsd()
            if not is_valid:
                for error in errors:
                    print(f"Validation error: {error}")
            ```
        """
        if lxml_etree is None:
            raise ImportError(
                "lxml is required for XSD validation. "
                "Install it with: pip install lxml"
            )

        xsd_path = self.get_xsd_path()
        if not xsd_path.exists():
            raise FileNotFoundError(f"XSD schema file not found: {xsd_path}")

        # Generate XML if not provided
        if xml_string is None:
            xml_string = self.generate_xml(validate=False)

        errors: List[str] = []

        try:
            # Parse the XSD schema
            with open(xsd_path, "rb") as xsd_file:
                xsd_doc = lxml_etree.parse(xsd_file)
                xsd_schema = lxml_etree.XMLSchema(xsd_doc)

            # Parse the XML
            xml_doc = lxml_etree.fromstring(xml_string.encode("utf-8"))

            # Validate
            is_valid: bool = xsd_schema.validate(xml_doc)

            if not is_valid:
                for error in xsd_schema.error_log:  # type: ignore[union-attr]
                    errors.append(f"Line {error.line}: {error.message}")  # type: ignore

            return is_valid, errors

        except lxml_etree.XMLSyntaxError as e:
            errors.append(f"XML syntax error: {e}")
            return False, errors
        except lxml_etree.XMLSchemaParseError as e:
            errors.append(f"XSD schema parse error: {e}")
            return False, errors

    def generate_and_validate_xml(self) -> str:
        """
        Generate XML and validate it against the XSD schema.

        Returns:
            str: Valid XML string

        Raises:
            ValueError: If XML validation fails
            ImportError: If lxml is not installed
            FileNotFoundError: If the XSD file is not found

        Example:
            ```python
            order_data = NormalOrderData(orders=[...])
            try:
                xml = order_data.generate_and_validate_xml()
                print("XML is valid!")
            except ValueError as e:
                print(f"Validation failed: {e}")
            ```
        """
        xml_string = self.generate_xml(validate=True)
        is_valid, errors = self.validate_against_xsd(xml_string)

        if not is_valid:
            error_msg = "XML validation against XSD failed:\n" + "\n".join(errors)
            raise ValueError(error_msg)

        return xml_string



"""Base types for Carlo TMS integration based on FOneOrderImport-v3.xsd."""

from typing import Optional, List
from datetime import datetime
from decimal import Decimal
from pydantic import BaseModel, Field

from .enums import ImportExportAction, LoadingUnitPostingsGeneration


class Country(BaseModel):
    """Country information."""

    iso_two_character_country_code: Optional[str] = Field(
        None, max_length=2, serialization_alias="IsoTwoCharacterCountryCode"
    )
    iso_three_character_country_code: Optional[str] = Field(
        None, max_length=3, serialization_alias="IsoThreeCharacterCountryCode"
    )
    iso_numeric_country_code: Optional[str] = Field(
        None, max_length=3, serialization_alias="IsoNumericCountryCode"
    )
    country_id: Optional[str] = Field(
        None, max_length=3, serialization_alias="CountryId"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Address(BaseModel):
    """Address information."""

    location1: Optional[str] = Field(
        None, max_length=70, serialization_alias="Location1"
    )
    location2: Optional[str] = Field(
        None, max_length=70, serialization_alias="Location2"
    )
    zip_code: Optional[str] = Field(None, max_length=10, serialization_alias="ZipCode")
    street: Optional[str] = Field(None, max_length=60, serialization_alias="Street")
    house_number: Optional[str] = Field(
        None, max_length=10, serialization_alias="HouseNumber"
    )
    country: Optional[Country] = Field(None, serialization_alias="Country")
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Weights(BaseModel):
    """Weight information."""

    effective_weight_in_kilogram: Optional[Decimal] = Field(
        None, serialization_alias="EffectiveWeightInKilogram"
    )
    freight_weight_in_kilogram: Optional[Decimal] = Field(
        None, serialization_alias="FreightWeightInKilogram"
    )
    carrier_weight_in_kilogram: Optional[Decimal] = Field(
        None, serialization_alias="CarrierWeightInKilogram"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Size(BaseModel):
    """Size dimensions."""

    length_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="LengthInMeters"
    )
    width_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="WidthInMeters"
    )
    height_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="HeightInMeters"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Dimensions(BaseModel):
    """Dimension information."""

    length_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="LengthInMeters"
    )
    width_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="WidthInMeters"
    )
    height_in_meters: Optional[Decimal] = Field(
        None, serialization_alias="HeightInMeters"
    )
    volume_in_cubic_meters: Optional[Decimal] = Field(
        None, serialization_alias="VolumeInCubicMeters"
    )


class Money(BaseModel):
    """Money value with amount and currency."""

    amount: Decimal = Field(..., serialization_alias="Amount")
    currency: Optional[str] = Field(None, serialization_alias="Currency")


class GeoCoordinate(BaseModel):
    """Geographic coordinates."""

    latitude: Decimal = Field(..., serialization_alias="Latitude")
    longitude: Decimal = Field(..., serialization_alias="Longitude")


class CustomTypeValue(BaseModel):
    """Custom type value."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=100, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class LoadingUnit(BaseModel):
    """Loading unit master data."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    external_number: Optional[str] = Field(
        None, max_length=35, serialization_alias="ExternalNumber"
    )
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=30, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentLoadingUnitGrai(BaseModel):
    """GRAI number for loading unit."""

    grai_number: Optional[str] = Field(
        None, max_length=50, serialization_alias="GraiNumber"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentLoadingUnitItem(BaseModel):
    """Loading unit item for consignment."""

    item_number: Optional[int] = Field(None, serialization_alias="ItemNumber")
    generate_loading_unit_postings: Optional[LoadingUnitPostingsGeneration] = Field(
        None, serialization_alias="GenerateLoadingUnitPostings"
    )
    grai_numbers: Optional[List["ConsignmentLoadingUnitGrai"]] = Field(
        None, serialization_alias="GraiNumbers"
    )
    quantity: Optional[int] = Field(None, serialization_alias="Quantity")
    loading_unit: Optional[LoadingUnit] = Field(None, serialization_alias="LoadingUnit")
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentTimes(BaseModel):
    """Times for consignment (loading and delivery dates)."""

    loading_date_start: Optional[datetime] = Field(
        None, serialization_alias="LoadingDateStart"
    )
    loading_date_end: Optional[datetime] = Field(
        None, serialization_alias="LoadingDateEnd"
    )
    delivery_date_start: Optional[datetime] = Field(
        None, serialization_alias="DeliveryDateStart"
    )
    delivery_date_end: Optional[datetime] = Field(
        None, serialization_alias="DeliveryDateEnd"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class AmountData(BaseModel):
    """Amount data for rates."""

    amount: Optional[Money] = Field(None, serialization_alias="Amount")
    currency: Optional[str] = Field(None, max_length=3, serialization_alias="Currency")
    info: Optional[str] = Field(None, max_length=255, serialization_alias="Info")
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class DangerousGoods(BaseModel):
    """Dangerous goods specification."""

    carrier_number: Optional[str] = Field(
        None, max_length=4, serialization_alias="CarrierNumber"
    )
    technical_name: Optional[str] = Field(
        None, max_length=2000, serialization_alias="TechnicalName"
    )
    additional_text: Optional[str] = Field(
        None, max_length=255, serialization_alias="AdditionalText"
    )
    hazard_class: Optional[str] = Field(
        None, max_length=10, serialization_alias="Class"
    )
    classification_code: Optional[str] = Field(
        None, max_length=10, serialization_alias="ClassificationCode"
    )
    packing_group: Optional[str] = Field(
        None, max_length=10, serialization_alias="PackingGroup"
    )
    un_number: Optional[str] = Field(
        None, max_length=10, serialization_alias="UnNumber"
    )
    are_danger_to_environment: Optional[bool] = Field(
        None, serialization_alias="AreDangerToEnvironment"
    )
    limited_amount: Optional[bool] = Field(None, serialization_alias="LimitedAmount")
    net_weight_in_kilogram: Optional[Decimal] = Field(
        None, serialization_alias="NetWeightInKilogram"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class SaleType(BaseModel):
    """Sales type."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    external_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="ExternalNumber"
    )
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class TrafficType(BaseModel):
    """Traffic type."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    external_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="ExternalNumber"
    )
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class TransportType(BaseModel):
    """Transport type."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class VehicleType(BaseModel):
    """Vehicle type."""

    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Currency(BaseModel):
    """Currency information."""

    matchcode: Optional[str] = Field(
        None, max_length=3, serialization_alias="Matchcode"
    )
    iso_currency_code: Optional[str] = Field(
        None, max_length=3, serialization_alias="IsoCurrencyCode"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ContainerType(BaseModel):
    """Container type."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class AreaRelation(BaseModel):
    """Area relation information."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ContraAccountGroup(BaseModel):
    """Contra account group for financial accounting."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=40, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class OrganizationalUnit(BaseModel):
    """Organizational unit."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    designation: Optional[str] = Field(
        None, max_length=80, serialization_alias="Designation"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Header(BaseModel):
    """Header information for the document."""

    send_date: datetime = Field(..., serialization_alias="SendDate")
    export_item_reference: str = Field(..., serialization_alias="ExportItemReference")



"""Enum types for Carlo TMS integration based on FOneOrderImport-v3.xsd."""

from enum import IntEnum


class ImportExportAction(IntEnum):
    """Import action tag for specifying create, update, updateorcreate or delete."""

    NONE = 0
    CREATE = 1
    UPDATE = 2
    DELETE = 3
    UPDATE_OR_CREATE = 4


class OrderContext(IntEnum):
    """Order context enumeration."""

    NORMAL = 0
    STORAGE = 10
    REMOVE_FROM_STORAGE = 20
    STORAGE_RELOCATION = 30
    NORMAL_STORAGE_MOVEMENT = 100
    GENERAL_ORDER_WITH_LOADING_UNIT = 200


class Incoterms(IntEnum):
    """Incoterms enumeration."""

    NONE = 0
    EXW = 1  # Ex Works
    FCA = 2  # Free Carrier
    CPT = 3  # Carriage Paid To
    CIP = 4  # Carriage and Insurance Paid To
    DAP = 5  # Delivered at Place
    DPU = 6  # Delivered at Place Unloaded
    DDP = 7  # Delivered Duty Paid
    FAS = 8  # Free Alongside Ship
    FOB = 9  # Free On Board
    CFR = 10  # Cost and Freight
    CIF = 11  # Cost, Insurance and Freight


class DeliveryTerms(IntEnum):
    """Delivery terms enumeration."""

    NONE = 0
    FREI_HAUS = 1  # Free House
    AB_WERK = 2  # Ex Works
    UNFREI = 3  # Freight Collect


class TransportDateType(IntEnum):
    """Transport date type enumeration."""

    NOT_SET = 0
    ON = 1  # Datum genau
    BY = 2  # Datum bis
    FROM = 3  # Datum ab
    BETWEEN = 4  # Datum zwischen
    NOT_BEFORE = 5  # nicht vor
    NOT_AFTER = 6  # nicht nach
    AS_SOON_AS_POSSIBLE = 7  # So bald wie möglich
    AT_THE_EARLIEST = 8  # Datum ab
    AT_THE_LATEST = 9  # Datum bis


class ConsignmentPaymentType(IntEnum):
    """Consignment payment type enumeration."""

    UNDEFINED = 0
    ADVANCE_DEBIT = 1  # Vorkasse
    INVOICE = 2  # Rechnung


class PaymentStatus(IntEnum):
    """Payment status enumeration."""

    UNKNOWN = 0
    UNPAID = 1
    PAID = 2
    INSTALLMENT = 3  # Teilzahlung


class ContainerLoadType(IntEnum):
    """Container load type enumeration."""

    NONE = 0
    FULL_CONTAINER_LOAD = 1  # FCL
    LESS_THAN_CONTAINER_LOAD = 2  # LCL
    BUYERS_CONSOLIDATION = 3  # BCL
    FULL_TRUCK_LOAD = 4  # FTL
    LESS_THAN_TRUCK_LOAD = 5  # LTL
    AIR_FREIGHT = 6


class InttraReferenceType(IntEnum):
    """Inttra reference type enumeration."""

    INTTRA_COMPANY_ID = 0  # Firmen-ID
    INTTRA_PARTNER_ALIAS = 1  # Partner-Alias
    DUNS_CODE = 2  # DUNS-Code


class HazardousMaterial(IntEnum):
    """Hazardous material enumeration."""

    NONE = 0  # Nein
    YES = 1  # Ja
    LIMITED_QUANTITIES = 2  # Begrenzte Menge
    EXCEPTED_QUANTITIES = 3  # Freigestellte Menge
    EXCEPTED_AND_LIMITED_QUANTITIES = 4  # Freigestellte + begrenzte Menge


class WaterHazardClass(IntEnum):
    """Water hazard class enumeration."""

    NONE = -1  # Nicht wassergefährdend
    NOT_SPECIFIED = 0  # (Nicht gesetzt)
    WGK_1 = 1  # schwach wassergefährdend
    WGK_2 = 2  # deutlich wassergefährdend
    WGK_3 = 3  # stark wassergefährdend


class MovementType(IntEnum):
    """Movement type enumeration for air/sea freight."""

    UNDEFINED = 0
    DOOR_TO_DOOR = 1  # D/D
    PORT_TO_DOOR = 2  # P/D
    DOOR_TO_PORT = 3  # D/P
    PORT_TO_PORT = 4  # P/P


class FreightType(IntEnum):
    """Freight type enumeration."""

    NONE = -1  # Nicht definiert
    ROAD_FREIGHT = 0  # Straße
    AIR_FREIGHT = 1  # Luftfracht
    SEA_FREIGHT = 2  # Seefracht


class RegulatedEntityCategory(IntEnum):
    """Regulated entity category for air freight security."""

    UNDEFINED = 0
    KNOWN_CONSIGNOR = 1
    AIRCRAFT_OPERATOR = 2
    REGULATED_AGENT = 3


class LoadingUnitPostingsGeneration(IntEnum):
    """Loading unit postings generation enumeration."""

    YES = 0  # Ja (Note: 0 = Yes in the XSD)
    NO = 1  # Nein
    DECIDE_LATER = 2  # Nicht definiert



"""Types for Carlo TMS integration based on FOneOrderImport-v3.xsd."""

from .enums import (
    ImportExportAction,
    OrderContext,
    Incoterms,
    DeliveryTerms,
    TransportDateType,
    ConsignmentPaymentType,
    PaymentStatus,
    ContainerLoadType,
    InttraReferenceType,
    HazardousMaterial,
    WaterHazardClass,
    MovementType,
    FreightType,
    RegulatedEntityCategory,
    LoadingUnitPostingsGeneration,
)

from .base import (
    Country,
    Address,
    Weights,
    Size,
    Dimensions,
    Money,
    GeoCoordinate,
    CustomTypeValue,
    LoadingUnit,
    ConsignmentLoadingUnitGrai,
    ConsignmentLoadingUnitItem,
    ConsignmentTimes,
    AmountData,
    DangerousGoods,
    SaleType,
    TrafficType,
    TransportType,
    VehicleType,
    Currency,
    ContainerType,
    AreaRelation,
    ContraAccountGroup,
    OrganizationalUnit,
    Header,
)

from .business_partner import (
    ContactPerson,
    BusinessPartner,
    ConsignmentContactPerson,
    ConsignmentBusinessPartner,
)

from .consignment import (
    ConsignmentCustomFields,
    ConsignmentAirAndSea,
    ConsignmentItem,
    Consignment,
)

__all__ = [
    # Enums
    "ImportExportAction",
    "OrderContext",
    "Incoterms",
    "DeliveryTerms",
    "TransportDateType",
    "ConsignmentPaymentType",
    "PaymentStatus",
    "ContainerLoadType",
    "InttraReferenceType",
    "HazardousMaterial",
    "WaterHazardClass",
    "MovementType",
    "FreightType",
    "RegulatedEntityCategory",
    "LoadingUnitPostingsGeneration",
    # Base types
    "Country",
    "Address",
    "Weights",
    "Size",
    "Dimensions",
    "Money",
    "GeoCoordinate",
    "CustomTypeValue",
    "LoadingUnit",
    "ConsignmentLoadingUnitGrai",
    "ConsignmentLoadingUnitItem",
    "ConsignmentTimes",
    "AmountData",
    "DangerousGoods",
    "SaleType",
    "TrafficType",
    "TransportType",
    "VehicleType",
    "Currency",
    "ContainerType",
    "AreaRelation",
    "ContraAccountGroup",
    "OrganizationalUnit",
    "Header",
    # Business partner types
    "ContactPerson",
    "BusinessPartner",
    "ConsignmentContactPerson",
    "ConsignmentBusinessPartner",
    # Consignment types
    "ConsignmentCustomFields",
    "ConsignmentAirAndSea",
    "ConsignmentItem",
    "Consignment",
]



"""Consignment types for Carlo TMS integration based on FOneOrderImport-v3.xsd."""

from typing import Optional, List
from datetime import datetime, time, date
from decimal import Decimal
from pydantic import BaseModel, Field

from .enums import (
    ImportExportAction,
    Incoterms,
    DeliveryTerms,
    TransportDateType,
    ConsignmentPaymentType,
    PaymentStatus,
    ContainerLoadType,
    HazardousMaterial,
    WaterHazardClass,
    FreightType,
    RegulatedEntityCategory,
    MovementType,
)
from .base import (
    Weights,
    Size,
    Dimensions,
    Money,
    ConsignmentTimes,
    ConsignmentLoadingUnitItem,
    DangerousGoods,
    AmountData,
    SaleType,
    TrafficType,
    TransportType,
    VehicleType,
    Address,
    AreaRelation,
    ContraAccountGroup,
    Currency,
    ContainerType,
    CustomTypeValue,
    Country,
)
from .business_partner import (
    BusinessPartner,
    ConsignmentBusinessPartner,
    ConsignmentContactPerson,
)


class ConsignmentCustomFields(BaseModel):
    """Custom fields for consignment."""

    custom_type_value1: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue1"
    )
    custom_type_value2: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue2"
    )
    custom_type_value3: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue3"
    )
    custom_type_value4: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue4"
    )
    custom_type_value5: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue5"
    )
    custom_type_value6: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue6"
    )
    custom_type_value7: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue7"
    )
    custom_type_value8: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue8"
    )
    custom_type_value9: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue9"
    )
    custom_type_value10: Optional[CustomTypeValue] = Field(
        None, serialization_alias="CustomTypeValue10"
    )
    custom_bool1: Optional[bool] = Field(None, serialization_alias="CustomBool1")
    custom_bool2: Optional[bool] = Field(None, serialization_alias="CustomBool2")
    custom_bool3: Optional[bool] = Field(None, serialization_alias="CustomBool3")
    custom_bool4: Optional[bool] = Field(None, serialization_alias="CustomBool4")
    custom_bool5: Optional[bool] = Field(None, serialization_alias="CustomBool5")
    custom_bool6: Optional[bool] = Field(None, serialization_alias="CustomBool6")
    custom_bool7: Optional[bool] = Field(None, serialization_alias="CustomBool7")
    custom_bool8: Optional[bool] = Field(None, serialization_alias="CustomBool8")
    custom_bool9: Optional[bool] = Field(None, serialization_alias="CustomBool9")
    custom_bool10: Optional[bool] = Field(None, serialization_alias="CustomBool10")
    custom_string1: Optional[str] = Field(
        None, max_length=255, serialization_alias="CustomString1"
    )
    custom_string2: Optional[str] = Field(
        None, max_length=255, serialization_alias="CustomString2"
    )
    custom_string3: Optional[str] = Field(
        None, max_length=255, serialization_alias="CustomString3"
    )
    custom_string4: Optional[str] = Field(
        None, max_length=255, serialization_alias="CustomString4"
    )
    custom_string5: Optional[str] = Field(
        None, max_length=255, serialization_alias="CustomString5"
    )
    custom_float1: Optional[Decimal] = Field(None, serialization_alias="CustomFloat1")
    custom_float2: Optional[Decimal] = Field(None, serialization_alias="CustomFloat2")
    custom_float3: Optional[Decimal] = Field(None, serialization_alias="CustomFloat3")
    custom_float4: Optional[Decimal] = Field(None, serialization_alias="CustomFloat4")
    custom_float5: Optional[Decimal] = Field(None, serialization_alias="CustomFloat5")
    custom_date_time1: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime1"
    )
    custom_date_time2: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime2"
    )
    custom_date_time3: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime3"
    )
    custom_date_time4: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime4"
    )
    custom_date_time5: Optional[datetime] = Field(
        None, serialization_alias="CustomDateTime5"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentAirAndSea(BaseModel):
    """Air and sea freight specific fields."""

    is_shipper_secure: Optional[bool] = Field(
        None, serialization_alias="IsShipperSecure"
    )
    transport_way: Optional[MovementType] = Field(
        None, serialization_alias="TransportWay"
    )
    freight_type: Optional[FreightType] = Field(None, serialization_alias="FreightType")
    regulated_entity_category: Optional[RegulatedEntityCategory] = Field(
        None, serialization_alias="RegulatedEntityCategory"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentItem(BaseModel):
    """Consignment item (position)."""

    item_number: Optional[int] = Field(None, serialization_alias="ItemNumber")
    quantity: Optional[Decimal] = Field(None, serialization_alias="Quantity")
    content1: Optional[str] = Field(
        None, max_length=2000, serialization_alias="Content1"
    )
    content2: Optional[str] = Field(
        None, max_length=2000, serialization_alias="Content2"
    )
    weights: Optional[Weights] = Field(None, serialization_alias="Weights")
    size: Optional[Size] = Field(None, serialization_alias="Size")
    storage_places: Optional[Decimal] = Field(None, serialization_alias="StoragePlaces")
    mark: Optional[str] = Field(None, max_length=255, serialization_alias="Mark")
    loading_unit_item: Optional[ConsignmentLoadingUnitItem] = Field(
        None, serialization_alias="LoadingUnitItem"
    )
    dangerous_goods: Optional[DangerousGoods] = Field(
        None, serialization_alias="DangerousGoods"
    )
    hazardous_material: Optional[HazardousMaterial] = Field(
        None, serialization_alias="HazardousMaterial"
    )
    packaging: Optional[str] = Field(
        None, max_length=10, serialization_alias="Packaging"
    )
    float1: Optional[Decimal] = Field(None, serialization_alias="Float1")
    float2: Optional[Decimal] = Field(None, serialization_alias="Float2")
    float3: Optional[Decimal] = Field(None, serialization_alias="Float3")
    float4: Optional[Decimal] = Field(None, serialization_alias="Float4")
    float5: Optional[Decimal] = Field(None, serialization_alias="Float5")
    custom_float6: Optional[Decimal] = Field(None, serialization_alias="CustomFloat6")
    custom_float7: Optional[Decimal] = Field(None, serialization_alias="CustomFloat7")
    custom_float8: Optional[Decimal] = Field(None, serialization_alias="CustomFloat8")
    custom_float9: Optional[Decimal] = Field(None, serialization_alias="CustomFloat9")
    custom_float10: Optional[Decimal] = Field(None, serialization_alias="CustomFloat10")
    best_before_date: Optional[datetime] = Field(
        None, serialization_alias="BestBeforeDate"
    )
    storage_period: Optional[datetime] = Field(
        None, serialization_alias="StoragePeriod"
    )
    temperature_in_celsius: Optional[Decimal] = Field(
        None, serialization_alias="TemperatureInCelsius"
    )
    temperature_variation_in_celsius: Optional[Decimal] = Field(
        None, serialization_alias="TemperatureVariationInCelsius"
    )
    water_hazard_class: Optional[WaterHazardClass] = Field(
        None, serialization_alias="WaterHazardClass"
    )
    origin_country: Optional[Country] = Field(None, serialization_alias="OriginCountry")
    commodity_tariff_code: Optional[str] = Field(
        None, max_length=20, serialization_alias="CommodityTariffCode"
    )
    net_mass: Optional[Decimal] = Field(None, serialization_alias="NetMass")
    article_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="ArticleNumber"
    )
    unit: Optional[str] = Field(None, max_length=10, serialization_alias="Unit")
    fzg_matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="FzgMatchcode"
    )
    fzg_description: Optional[str] = Field(
        None, max_length=256, serialization_alias="FzgDescription"
    )
    ean_code: Optional[str] = Field(None, max_length=35, serialization_alias="EanCode")
    price_per_unit: Optional[Money] = Field(None, serialization_alias="PricePerUnit")
    total_price: Optional[Money] = Field(None, serialization_alias="TotalPrice")
    article_quantity: Optional[Decimal] = Field(
        None, serialization_alias="ArticleQuantity"
    )
    manufacturer: Optional[str] = Field(
        None, max_length=255, serialization_alias="Manufacturer"
    )
    currency_matchcode: Optional[str] = Field(
        None, max_length=3, serialization_alias="CurrencyMatchcode"
    )
    iso_currency_code: Optional[str] = Field(
        None, max_length=3, serialization_alias="IsoCurrencyCode"
    )
    batch_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="BatchNumber"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class Consignment(BaseModel):
    """Consignment / transport order."""

    item_number: Optional[int] = Field(None, serialization_alias="ItemNumber")
    external_number: Optional[str] = Field(
        None, max_length=50, serialization_alias="ExternalNumber"
    )
    sender: Optional[ConsignmentBusinessPartner] = Field(
        None, serialization_alias="Sender"
    )
    sender_contact_person: Optional[ConsignmentContactPerson] = Field(
        None, serialization_alias="SenderContactPerson"
    )
    receiver: Optional[ConsignmentBusinessPartner] = Field(
        None, serialization_alias="Receiver"
    )
    receiver_contact_person: Optional[ConsignmentContactPerson] = Field(
        None, serialization_alias="ReceiverContactPerson"
    )
    different_loading_point: Optional[ConsignmentBusinessPartner] = Field(
        None, serialization_alias="DifferentLoadingPoint"
    )
    different_delivery_point: Optional[ConsignmentBusinessPartner] = Field(
        None, serialization_alias="DifferentDeliveryPoint"
    )
    loading_unit_items: Optional[List[ConsignmentLoadingUnitItem]] = Field(
        None, serialization_alias="LoadingUnitItems"
    )
    consignment_items: Optional[List[ConsignmentItem]] = Field(
        None, serialization_alias="ConsignmentItems"
    )
    times: Optional[ConsignmentTimes] = Field(None, serialization_alias="Times")
    flat_rate_customer: Optional[AmountData] = Field(
        None, serialization_alias="FlatRateCustomer"
    )
    flat_rate_receiver: Optional[AmountData] = Field(
        None, serialization_alias="FlatRateReceiver"
    )
    incoterms: Optional[Incoterms] = Field(None, serialization_alias="Incoterms")
    delivery_terms: Optional[DeliveryTerms] = Field(
        None, serialization_alias="DeliveryTerms"
    )
    sale_type: Optional[SaleType] = Field(None, serialization_alias="SaleType")
    traffic_type: Optional[TrafficType] = Field(None, serialization_alias="TrafficType")
    transport_type: Optional[TransportType] = Field(
        None, serialization_alias="TransportType"
    )
    execution_date: Optional[date] = Field(None, serialization_alias="ExecutionDate")
    pickup: Optional[bool] = Field(None, serialization_alias="Pickup")
    pickup_splitting_business_partner: Optional[BusinessPartner] = Field(
        None, serialization_alias="PickupSplittingBusinessPartner"
    )
    default_carrier: Optional[BusinessPartner] = Field(
        None, serialization_alias="DefaultCarrier"
    )
    sender_terminal: Optional[BusinessPartner] = Field(
        None, serialization_alias="SenderTerminal"
    )
    receiver_terminal: Optional[BusinessPartner] = Field(
        None, serialization_alias="ReceiverTerminal"
    )
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    container_number: Optional[str] = Field(
        None, max_length=40, serialization_alias="ContainerNumber"
    )
    loading_date_type: Optional[TransportDateType] = Field(
        None, serialization_alias="LoadingDateType"
    )
    delivery_date_type: Optional[TransportDateType] = Field(
        None, serialization_alias="DeliveryDateType"
    )
    initial_costs_tax_free: Optional[Money] = Field(
        None, serialization_alias="InitialCostsTaxFree"
    )
    initial_costs_taxable: Optional[Money] = Field(
        None, serialization_alias="InitialCostsTaxable"
    )
    cash_on_delivery_gross_amount: Optional[Money] = Field(
        None, serialization_alias="CashOnDeliveryGrossAmount"
    )
    cash_on_delivery_tax_free: Optional[Money] = Field(
        None, serialization_alias="CashOnDeliveryTaxFree"
    )
    cash_on_delivery_taxable: Optional[Money] = Field(
        None, serialization_alias="CashOnDeliveryTaxable"
    )
    cash_on_delivery_value_added_tax: Optional[Money] = Field(
        None, serialization_alias="CashOnDeliveryValueAddedTax"
    )
    cash_on_delivery_value_added_tax_rate_in_percent: Optional[Decimal] = Field(
        None, serialization_alias="CashOnDeliveryValueAddedTaxRateInPercent"
    )
    cash_on_delivery_currency: Optional[Currency] = Field(
        None, serialization_alias="CashOnDeliveryCurrency"
    )
    value_of_goods: Optional[Money] = Field(None, serialization_alias="ValueOfGoods")
    value_of_goods_currency: Optional[Currency] = Field(
        None, serialization_alias="ValueOfGoodsCurrency"
    )
    payment_type: Optional[ConsignmentPaymentType] = Field(
        None, serialization_alias="PaymentType"
    )
    payment_status: Optional[PaymentStatus] = Field(
        None, serialization_alias="PaymentStatus"
    )
    fin_acc_cost_center1: Optional[str] = Field(
        None, max_length=20, serialization_alias="FinAccCostCenter1"
    )
    fin_acc_cost_center2: Optional[str] = Field(
        None, max_length=20, serialization_alias="FinAccCostCenter2"
    )
    fin_acc_debtor_contra_account_group: Optional[ContraAccountGroup] = Field(
        None, serialization_alias="FinAccDebtorContraAccountGroup"
    )
    fin_acc_creditor_contra_account_group: Optional[ContraAccountGroup] = Field(
        None, serialization_alias="FinAccCreditorContraAccountGroup"
    )
    named_location: Optional[Address] = Field(None, serialization_alias="NamedLocation")
    kilometers: Optional[Decimal] = Field(None, serialization_alias="Kilometers")
    quantity: Optional[Decimal] = Field(None, serialization_alias="Quantity")
    weights: Optional[Weights] = Field(None, serialization_alias="Weights")
    dimensions: Optional[Dimensions] = Field(None, serialization_alias="Dimensions")
    required_storage_places: Optional[Decimal] = Field(
        None, serialization_alias="RequiredStoragePlaces"
    )
    timeframe: Optional[time] = Field(None, serialization_alias="Timeframe")
    named_place: Optional[str] = Field(
        None, max_length=40, serialization_alias="NamedPlace"
    )
    additional_agreement: Optional[str] = Field(
        None, max_length=40, serialization_alias="AdditionalAgreement"
    )
    calculation_default: Optional[str] = Field(
        None, max_length=10, serialization_alias="CalculationDefault"
    )
    flat_rate_carrier: Optional[AmountData] = Field(
        None, serialization_alias="FlatRateCarrier"
    )
    vehicle_type: Optional[VehicleType] = Field(None, serialization_alias="VehicleType")
    sender_relation: Optional[AreaRelation] = Field(
        None, serialization_alias="SenderRelation"
    )
    receiver_relation: Optional[AreaRelation] = Field(
        None, serialization_alias="ReceiverRelation"
    )
    abs_emp_relation: Optional[AreaRelation] = Field(
        None, serialization_alias="AbsEmpRelation"
    )
    maximum_size: Optional[Size] = Field(None, serialization_alias="MaximumSize")
    maximum_weight_in_kilogram: Optional[Decimal] = Field(
        None, serialization_alias="MaximumWeightInKilogram"
    )
    loading_duration_in_minutes: Optional[int] = Field(
        None, serialization_alias="LoadingDurationInMinutes"
    )
    unloading_duration_in_minutes: Optional[int] = Field(
        None, serialization_alias="UnloadingDurationInMinutes"
    )
    contains_dangerous_goods: Optional[bool] = Field(
        None, serialization_alias="ContainsDangerousGoods"
    )
    custom_fields: Optional[ConsignmentCustomFields] = Field(
        None, serialization_alias="CustomFields"
    )
    is_heavy_duty_transport: Optional[bool] = Field(
        None, serialization_alias="IsHeavyDutyTransport"
    )
    air_and_sea: Optional[ConsignmentAirAndSea] = Field(
        None, serialization_alias="AirAndSea"
    )
    container_type: Optional[ContainerType] = Field(
        None, serialization_alias="ContainerType"
    )
    load_type: Optional[ContainerLoadType] = Field(None, serialization_alias="LoadType")
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")



"""Business partner types for Carlo TMS integration based on FOneOrderImport-v3.xsd."""

from typing import Optional, List
from pydantic import BaseModel, Field

from .enums import ImportExportAction, InttraReferenceType
from .base import Address


class ContactPerson(BaseModel):
    """Contact person from master data."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    first_name: Optional[str] = Field(
        None, max_length=20, serialization_alias="FirstName"
    )
    last_name: Optional[str] = Field(
        None, max_length=20, serialization_alias="LastName"
    )
    phone_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="PhoneNumber"
    )
    mobile_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="MobileNumber"
    )
    fax_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="FaxNumber"
    )
    email_address: Optional[str] = Field(
        None, max_length=255, serialization_alias="EmailAddress"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class BusinessPartner(BaseModel):
    """Business partner from CarLo master data."""

    number: Optional[int] = Field(None, serialization_alias="Number")
    external_number: Optional[str] = Field(
        None, max_length=20, serialization_alias="ExternalNumber"
    )
    matchcode: Optional[str] = Field(
        None, max_length=10, serialization_alias="Matchcode"
    )
    global_location_number: Optional[str] = Field(
        None, max_length=50, serialization_alias="GlobalLocationNumber"
    )
    name1: Optional[str] = Field(None, max_length=40, serialization_alias="Name1")
    name2: Optional[str] = Field(None, max_length=40, serialization_alias="Name2")
    name3: Optional[str] = Field(None, max_length=40, serialization_alias="Name3")
    name4: Optional[str] = Field(None, max_length=40, serialization_alias="Name4")
    phone_number_head_office: Optional[str] = Field(
        None, max_length=30, serialization_alias="PhoneNumberHeadOffice"
    )
    contact_persons: Optional[List[ContactPerson]] = Field(
        None, serialization_alias="ContactPersons"
    )
    appendix: Optional[str] = Field(None, max_length=35, serialization_alias="Appendix")
    fin_acc_account_debtor: Optional[str] = Field(
        None, max_length=20, serialization_alias="FinAccAccountDebtor"
    )
    fin_acc_account_creditor: Optional[str] = Field(
        None, max_length=20, serialization_alias="FinAccAccountCreditor"
    )
    inttra_ref_number: Optional[str] = Field(
        None, max_length=50, serialization_alias="InttraRefNumber"
    )
    inttra_ref_type: Optional[InttraReferenceType] = Field(
        None, serialization_alias="InttraRefType"
    )
    home_edit_address: Optional[Address] = Field(
        None, serialization_alias="HomeEditAddress"
    )
    sender_receiver: Optional[bool] = Field(None, serialization_alias="SenderReceiver")
    freight_business_partner: Optional[bool] = Field(
        None, serialization_alias="FreightBusinessPartner"
    )
    value_added_tax_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="ValueAddedTaxNumber"
    )
    vat_iso_two_character_country_code: Optional[str] = Field(
        None, max_length=2, serialization_alias="VATIsoTwoCharacterCountryCode"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentContactPerson(BaseModel):
    """Free contact person for consignment."""

    first_name: Optional[str] = Field(
        None, max_length=20, serialization_alias="FirstName"
    )
    last_name: Optional[str] = Field(
        None, max_length=20, serialization_alias="LastName"
    )
    phone_number: Optional[str] = Field(
        None, max_length=30, serialization_alias="PhoneNumber"
    )
    email_address: Optional[str] = Field(
        None, max_length=255, serialization_alias="EmailAddress"
    )
    master_data_contact_person: Optional[ContactPerson] = Field(
        None, serialization_alias="MasterDataContactPerson"
    )
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")


class ConsignmentBusinessPartner(BaseModel):
    """Free consignment business partner (Sender/Receiver/etc.)."""

    master_data_business_partner: Optional[BusinessPartner] = Field(
        None, serialization_alias="MasterDataBusinessPartner"
    )
    name1: Optional[str] = Field(None, max_length=40, serialization_alias="Name1")
    name2: Optional[str] = Field(None, max_length=40, serialization_alias="Name2")
    street: Optional[str] = Field(None, max_length=60, serialization_alias="Street")
    house_number: Optional[str] = Field(
        None, max_length=10, serialization_alias="HouseNumber"
    )
    country: Optional[str] = Field(
        None, max_length=2, serialization_alias="Country"
    )  # ISO 2-char country code
    zip_code: Optional[str] = Field(None, max_length=10, serialization_alias="ZipCode")
    city1: Optional[str] = Field(None, max_length=70, serialization_alias="City1")
    city2: Optional[str] = Field(None, max_length=70, serialization_alias="City2")
    action: Optional[ImportExportAction] = Field(None, serialization_alias="Action")
